@@ -1,6 +1,8 @@
+# -*- coding: utf-8 -*-
 # pylint: disable=W0612,E1101
 
 from datetime import datetime
+from inspect import getargspec
 import operator
 import nose
 
@@ -8,6 +10,7 @@ import numpy as np
 
 from pandas import Series, DataFrame, Index, isnull, notnull, pivot, MultiIndex
 from pandas.core.datetools import bday
+from pandas.core.nanops import nanall, nanany
 from pandas.core.panel import Panel
 from pandas.core.series import remove_na
 import pandas.core.common as com
@@ -169,6 +172,11 @@ class SafeForLongAndSparse(object):
 
         self.assertRaises(Exception, f, axis=obj.ndim)
 
+        # Unimplemented numeric_only parameter.
+        if 'numeric_only' in getargspec(f).args:
+            self.assertRaisesRegexp(NotImplementedError, name, f,
+                                    numeric_only=True)
+
 
 class SafeForSparse(object):
     _multiprocess_can_split_ = True
@@ -880,6 +888,21 @@ class TestPanel(tm.TestCase, PanelTests, CheckIndexing,
         wp = Panel(vals, copy=True)
         self.assertIsNot(wp.values, vals)
 
+        # GH #8285, test when scalar data is used to construct a Panel
+        # if dtype is not passed, it should be inferred
+        value_and_dtype = [(1, 'int64'), (3.14, 'float64'), ('foo', np.object_)]
+        for (val, dtype) in value_and_dtype:
+            wp = Panel(val, items=range(2), major_axis=range(3), minor_axis=range(4))
+            vals = np.empty((2, 3, 4), dtype=dtype)
+            vals.fill(val)
+            assert_panel_equal(wp, Panel(vals, dtype=dtype))
+
+        # test the case when dtype is passed
+        wp = Panel(1, items=range(2), major_axis=range(3), minor_axis=range(4), dtype='float32')
+        vals = np.empty((2, 3, 4), dtype='float32')
+        vals.fill(1)
+        assert_panel_equal(wp, Panel(vals, dtype='float32'))
+
     def test_constructor_cast(self):
         zero_filled = self.panel.fillna(0)
 
@@ -1495,6 +1518,18 @@ class TestPanel(tm.TestCase, PanelTests, CheckIndexing,
         # Previously, this was mutating the underlying index and changing its name
         assert_frame_equal(wp['bool'], panel['bool'], check_names=False)
 
+        # GH 8704
+        # with categorical
+        df = panel.to_frame()
+        df['category'] = df['str'].astype('category')
+
+        # to_panel
+        # TODO: this converts back to object
+        p = df.to_panel()
+        expected = panel.copy()
+        expected['category'] = 'foo'
+        assert_panel_equal(p,expected)
+
     def test_to_frame_multi_major(self):
         idx = MultiIndex.from_tuples([(1, 'one'), (1, 'two'), (2, 'one'),
                                       (2, 'two')])
@@ -2084,6 +2119,24 @@ class TestPanel(tm.TestCase, PanelTests, CheckIndexing,
         np.testing.assert_raises(Exception, pan.update, *(pan,),
                                  **{'raise_conflict': True})
 
+    def test_all_any(self):
+        self.assertTrue((self.panel.all(axis=0).values ==
+                         nanall(self.panel, axis=0)).all())
+        self.assertTrue((self.panel.all(axis=1).values ==
+                         nanall(self.panel, axis=1).T).all())
+        self.assertTrue((self.panel.all(axis=2).values ==
+                         nanall(self.panel, axis=2).T).all())
+        self.assertTrue((self.panel.any(axis=0).values ==
+                         nanany(self.panel, axis=0)).all())
+        self.assertTrue((self.panel.any(axis=1).values ==
+                         nanany(self.panel, axis=1).T).all())
+        self.assertTrue((self.panel.any(axis=2).values ==
+                         nanany(self.panel, axis=2).T).all())
+
+    def test_all_any_unhandled(self):
+        self.assertRaises(NotImplementedError, self.panel.all, bool_only=True)
+        self.assertRaises(NotImplementedError, self.panel.any, bool_only=True)
+
 
 class TestLongPanel(tm.TestCase):
     """
